@@ -65,6 +65,15 @@ bs_theme_dependencies <- function(
   theme <- as_bs_theme(theme)
   version <- theme_version(theme)
 
+  if (isTRUE(version >= 5)) {
+    shiny_version <- "1.6.0.9001"
+    msg <- sprintf("`bs_theme(version = 5)` is designed to work with shiny %s or higher", shiny_version)
+    if (isNamespaceLoaded("shiny") && !is_available("shiny", shiny_version)) warning(msg, call. = FALSE)
+    setHook(packageEvent("shiny", "onLoad"), function(...) {
+      if (!is_available("shiny", shiny_version)) warning(msg, call. = FALSE)
+    })
+  }
+
   if (is.character(cache)) {
     cache <- sass_cache_get_dir(cache)
   }
@@ -178,7 +187,7 @@ bs_theme_dependencies <- function(
 #'   [sass::sass_partial()].
 #' @inheritParams htmltools::htmlDependency
 #' @references
-#' <https://rstudio.github.io/bslib/articles/theming.html#themable-components-1>
+#' <https://rstudio.github.io/bslib/articles/custom-components.html>
 #'
 #'
 #' @return `bs_dependency()` returns an [htmltools::htmlDependency()] and
@@ -342,7 +351,7 @@ as_bs_theme <- function(theme) {
     # Also support `bs_theme_dependencies(version = '4')` and
     # `bs_theme_dependencies(theme = 'bootswatch')`
     if (length(theme) == 1) {
-      if (theme %in% c("4", "4-3", "4+3", "3")) {
+      if (theme %in% c(versions(), "4-3", "4+3")) {
         return(bs_theme(version = theme))
       } else {
         return(bs_theme(bootswatch = theme))
@@ -366,4 +375,3 @@ register_theme_dependency <- function(x) {
   getFromNamespace("registerThemeDependency", "shiny")(x)
 }
 
-
